// sample application to test the CAStar2DGrid class

#include "stdafx.h"
#include "AStarTestApp.h"
#include "AStar2DGrid.h"			// CAStar2DGrid
#include <assert.h>						// assert()
#include <math.h>							// sqrt()


//! to visualze the map
void Debug( HDC hdc, CAStar2DGrid &ref )
{
	for(int iY=-1;iY<(int)ref.GetHeight()+1;++iY)
	for(int iX=-1;iX<(int)ref.GetWidth()+1;++iX)
	{
		DWORD col = 0;								// border=obstacle
		int iBorder=0;

		CAStar2DGrid::SArrayPos v(iX,iY);

		if(iX>=0 && iX<(int)ref.GetWidth())
		if(iY>=0 && iY<(int)ref.GetHeight())
		{
			if(ref.IsGoal(v))
				col = 0x00ff00;								// goal
			else if(ref.GetObstacle(v))
				col = 0x000000;								// obstacle
			else if(ref.WasNodeTested(v))
			{
				col = 0x808080;iBorder=1;
			}
			else continue;
		}

		for(int y=iBorder;y<10-iBorder;++y)
		for(int x=iBorder;x<10-iBorder;++x)
			SetPixel(hdc,(iX+1)*10+x,(iY+1)*10+y,col);
	}
}


//! to visualize the path back to the start node
void DebugPath( HDC hdc, CAStar2DGrid &rGrid, CAStar2DGrid::SArrayPos v )
{
	OutputDebugString("Path found (backward): ");	

	while(v)
	{
		char str[256];

		sprintf_s(str,sizeof(str),"(%d,%d) ",v.m_X,v.m_Y);
		OutputDebugString(str);

		for(int y=2;y<8;++y)
		for(int x=2;x<8;++x)
			SetPixel(hdc,(v.m_X+1)*10+x,(v.m_Y+1)*10+y,0x00ff80);

		v = rGrid.GetPrevious(v);
	}

	OutputDebugString("\n");
}


// test function
void Draw( HDC hdc )
{
	CAStar2DGrid Grid(10,10);

	Grid.SetGoal(CAStar2DGrid::SArrayPos(6,9));						// define goal

	for(unsigned int dwY=4;dwY<=5;++dwY)
	for(unsigned int dwX=3;dwX<=9;++dwX)
		Grid.SetObstacle(CAStar2DGrid::SArrayPos(dwX,dwY));

	Grid.ResetAndSetStart(CAStar2DGrid::SArrayPos(3,1));		// define start

	Debug(hdc,Grid);																				// draw map before compuation

	CAStar2DGrid::SArrayPos p = Grid.Solve();

	Debug(hdc,Grid);																				// draw map with all evaluated node

	DebugPath(hdc,Grid,p);																	// draw path
}





// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------

// the following code is only needed to get some base application to run the sample

// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------






const TCHAR *szWindowClass = "ASTARTESTAPP";			// the main window class name

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;

	switch (message)
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);

		switch (wmId)
		{
			case IDM_EXIT:
				DestroyWindow(hWnd);
				break;
			default:
				return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		Draw(hdc);
		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}



int APIENTRY _tWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow )
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

	MSG msg;

	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ASTARTESTAPP));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDC_ASTARTESTAPP);
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

	RegisterClassEx(&wcex);

	HWND hWnd = CreateWindow(szWindowClass, "AStarTestApp", WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

	if(!hWnd)
		return FALSE;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);


	while(GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int)msg.wParam;
}